@@ -2,7 +2,6 @@
 
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE CPP #-}
 
 module Hledger.UI.AccountsScreen
  (accountsScreen
@@ -19,9 +18,6 @@ import Control.Monad
 import Control.Monad.IO.Class (liftIO)
 import Data.List
 import Data.Maybe
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Monoid ((<>))
-#endif
 import qualified Data.Text as T
 import Data.Time.Calendar (Day, addDays)
 import qualified Data.Vector as V
@@ -53,7 +49,7 @@ accountsScreen = AccountsScreen{
 
 asInit :: Day -> Bool -> UIState -> UIState
 asInit d reset ui@UIState{
-  aopts=UIOpts{cliopts_=CliOpts{reportspec_=rspec@ReportSpec{rsOpts=ropts}}},
+  aopts=UIOpts{cliopts_=copts@CliOpts{reportspec_=rspec@ReportSpec{rsOpts=ropts}}},
   ajournal=j,
   aScreen=s@AccountsScreen{}
   } =
@@ -81,7 +77,7 @@ asInit d reset ui@UIState{
                         as = map asItemAccountName displayitems
 
     -- Further restrict the query based on the current period and future/forecast mode.
-    rspec' = rspec{rsQuery=simplifyQuery $ And [rsQuery rspec, periodq, excludeforecastq (forecast_ ropts)]}
+    rspec' = rspec{rsQuery=simplifyQuery $ And [rsQuery rspec, periodq, excludeforecastq (forecast_ $ inputopts_ copts)]}
       where
         periodq = Date $ periodAsDateSpan $ period_ ropts
         -- Except in forecast mode, exclude future/forecast transactions.
@@ -100,11 +96,8 @@ asInit d reset ui@UIState{
       AccountsScreenItem{asItemIndentLevel        = indent
                         ,asItemAccountName        = fullacct
                         ,asItemDisplayAccountName = replaceHiddenAccountsNameWith "All" $ if tree_ ropts then shortacct else fullacct
-                        ,asItemRenderedAmounts    = map showAmountWithoutPrice amts
+                        ,asItemMixedAmount        = Just bal
                         }
-      where
-        Mixed amts = normaliseMixedAmountSquashPricesForDisplay $ stripPrices bal
-        stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=Nothing}
     displayitems = map displayitem items
     -- blanks added for scrolling control, cf RegisterScreen.
     -- XXX Ugly. Changing to 0 helps when debugging.
@@ -112,7 +105,7 @@ asInit d reset ui@UIState{
       AccountsScreenItem{asItemIndentLevel        = 0
                         ,asItemAccountName        = ""
                         ,asItemDisplayAccountName = ""
-                        ,asItemRenderedAmounts    = []
+                        ,asItemMixedAmount        = Nothing
                         }
 
 
@@ -124,10 +117,10 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportspec_=rspec}}
               ,aScreen=s@AccountsScreen{}
               ,aMode=mode
               } =
-  case mode of
-    Help       -> [helpDialog copts, maincontent]
-    -- Minibuffer e -> [minibuffer e, maincontent]
-    _          -> [maincontent]
+    case mode of
+      Help       -> [helpDialog copts, maincontent]
+      -- Minibuffer e -> [minibuffer e, maincontent]
+      _          -> [maincontent]
   where
     maincontent = Widget Greedy Greedy $ do
       c <- getContext
@@ -137,32 +130,24 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportspec_=rspec}}
           c^.availWidthL
           - 2 -- XXX due to margin ? shouldn't be necessary (cf UIUtils)
         displayitems = s ^. asList . listElementsL
-        maxacctwidthseen =
-          -- ltrace "maxacctwidthseen" $
-          V.maximum $
-          V.map (\AccountsScreenItem{..} -> asItemIndentLevel + Hledger.Cli.textWidth asItemDisplayAccountName) $
-          -- V.filter (\(indent,_,_,_) -> (indent-1) <= fromMaybe 99999 mdepth) $
-          displayitems
-        maxbalwidthseen =
-          -- ltrace "maxbalwidthseen" $
-          V.maximum $ V.map (\AccountsScreenItem{..} -> sum (map strWidth asItemRenderedAmounts) + 2 * (length asItemRenderedAmounts - 1)) displayitems
-        maxbalwidth =
-          -- ltrace "maxbalwidth" $
-          max 0 (availwidth - 2 - 4) -- leave 2 whitespace plus least 4 for accts
-        balwidth =
-          -- ltrace "balwidth" $
-          min maxbalwidth maxbalwidthseen
-        maxacctwidth =
-          -- ltrace "maxacctwidth" $
-          availwidth - 2 - balwidth
-        acctwidth =
-          -- ltrace "acctwidth" $
-          min maxacctwidth maxacctwidthseen
 
-        -- XXX how to minimise the balance column's jumping around
-        -- as you change the depth limit ?
+        acctwidths = V.map (\AccountsScreenItem{..} -> asItemIndentLevel + Hledger.Cli.textWidth asItemDisplayAccountName) displayitems
+        balwidths  = V.map (maybe 0 (wbWidth . showMixedAmountB oneLine) . asItemMixedAmount) displayitems
+        preferredacctwidth = V.maximum acctwidths
+        totalacctwidthseen = V.sum acctwidths
+        preferredbalwidth  = V.maximum balwidths
+        totalbalwidthseen  = V.sum balwidths
 
-        colwidths = (acctwidth, balwidth)
+        totalwidthseen = totalacctwidthseen + totalbalwidthseen
+        shortfall = preferredacctwidth + preferredbalwidth + 2 - availwidth
+        acctwidthproportion = fromIntegral totalacctwidthseen / fromIntegral totalwidthseen
+        adjustedacctwidth = min preferredacctwidth . max 15 . round $ acctwidthproportion * fromIntegral (availwidth - 2)  -- leave 2 whitespace for padding
+        adjustedbalwidth  = availwidth - 2 - adjustedacctwidth
+
+        -- XXX how to minimise the balance column's jumping around as you change the depth limit ?
+
+        colwidths | shortfall <= 0 = (preferredacctwidth, preferredbalwidth)
+                  | otherwise      = (adjustedacctwidth, adjustedbalwidth)
 
       render $ defaultLayout toplabel bottomlabel $ renderList (asDrawItem colwidths) True (_asList s)
 
@@ -178,7 +163,7 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportspec_=rspec}}
           <+> borderQueryStr (T.unpack . T.unwords . map textQuoteIfNeeded $ querystring_ ropts)
           <+> borderDepthStr mdepth
           <+> str (" ("++curidx++"/"++totidx++")")
-          <+> (if ignore_assertions_ $ inputopts_ copts
+          <+> (if ignore_assertions_ . balancingopts_ $ inputopts_ copts
                then withAttr ("border" <> "query") (str " ignoring balance assertions")
                else str "")
           where
@@ -213,7 +198,7 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportspec_=rspec}}
               -- ,("l", str "list")
               ,("-+", str "depth")
               ,("H", renderToggle (not ishistorical) "end-bals" "changes")
-              ,("F", renderToggle1 (isJust $ forecast_ ropts) "forecast")
+              ,("F", renderToggle1 (isJust . forecast_ $ inputopts_ copts) "forecast")
               --,("/", "filter")
               --,("DEL", "unfilter")
               --,("ESC", "cancel/top")
@@ -230,24 +215,17 @@ asDrawItem (acctwidth, balwidth) selected AccountsScreenItem{..} =
     -- c <- getContext
       -- let showitem = intercalate "\n" . balanceReportItemAsText defreportopts fmt
     render $
-      addamts asItemRenderedAmounts $
-      str (T.unpack $ fitText (Just acctwidth) (Just acctwidth) True True $ T.replicate (asItemIndentLevel) " " <> asItemDisplayAccountName) <+>
-      str "  " <+>
-      str (balspace asItemRenderedAmounts)
+      txt (fitText (Just acctwidth) (Just acctwidth) True True $ T.replicate (asItemIndentLevel) " " <> asItemDisplayAccountName) <+>
+      txt balspace <+>
+      splitAmounts balBuilder
       where
-        balspace as = replicate n ' '
-          where n = max 0 (balwidth - (sum (map strWidth as) + 2 * (length as - 1)))
-        addamts :: [String] -> Widget Name -> Widget Name
-        addamts [] w = w
-        addamts [a] w = (<+> renderamt a) w
-        -- foldl' :: (b -> a -> b) -> b -> t a -> b
-        -- foldl' (Widget -> String -> Widget) -> Widget -> [String] -> Widget
-        addamts (a:as) w = foldl' addamt (addamts [a] w) as
-        addamt :: Widget Name -> String -> Widget Name
-        addamt w a = ((<+> renderamt a) . (<+> str ", ")) w
-        renderamt :: String -> Widget Name
-        renderamt a | '-' `elem` a = withAttr (sel $ "list" <> "balance" <> "negative") $ str a
-                    | otherwise    = withAttr (sel $ "list" <> "balance" <> "positive") $ str a
+        balBuilder = maybe mempty showamt asItemMixedAmount
+        showamt = showMixedAmountB oneLine{displayMinWidth=Just balwidth, displayMaxWidth=Just balwidth}
+        balspace = T.replicate (2 + balwidth - wbWidth balBuilder) " "
+        splitAmounts = foldr1 (<+>) . intersperse (str ", ") . map renderamt . T.splitOn ", " . wbToText
+        renderamt :: T.Text -> Widget Name
+        renderamt a | T.any (=='-') a = withAttr (sel $ "list" <> "balance" <> "negative") $ txt a
+                    | otherwise       = withAttr (sel $ "list" <> "balance" <> "positive") $ txt a
         sel | selected  = (<> "selected")
             | otherwise = id
 
